# ----------------------------------------------------------------------------#
# Object::AlreadyBound                                                        #
#                                                                             #
# Copyright (c) 2001-02 Arun Kumar U <u_arunkumar@yahoo.com>.                 #
# All rights reserved.                                                        #
#                                                                             #
# This program is free software; you can redistribute it and/or               #
# modify it under the same terms as Perl itself.                              #
# ----------------------------------------------------------------------------#

package Object::AlreadyBound;

use Error;
use base qw( Error::Simple );

1;
